@@ -14,12 +14,13 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
-# Rewritten from scratch, but based on the code from gnome-schedule by:
-# - Philip Van Hoof <me at pvanhoof dot be>
-# - Gaute Hope <eg at gaute dot vetsj dot com>
-# - Kristof Vansant <de_lupus at pandora dot be>
+# Ideas from gnome-schedule: Philip Van Hoof, Gaute Hope, Kristof Vansant
 #
-EXAMPLE_USE = """
+# REQUEST: Please do NOT simply copy and paste this code into your own
+#          projects. Please package this module for your distribution and
+#          use as a direct dependancy.
+# 
+"""
 from crontab import CronTab
 import sys
 
@@ -34,6 +35,9 @@ job.dow.on('SUN')
 job.month.during('APR', 'JUN')
 job.month.also.during('OCT', 'DEC')
 
+job.every(2).days()
+job.setall(1, 12, None, None, None)
+
 job2 = cron.new(command='/foo/bar',comment='SomeID')
 job2.every_reboot()
 
@@ -60,8 +64,9 @@ for job7 in cron:
     sys.stdout.write(job7)
     job7.every().dow()
 
-cron.remove_all('echo')
-cron.remove_all('/foo/bar')
+cron.remove_all(command='/foo/bar')
+cron.remove_all(comment='This command')
+cron.remove_all()
 cron.write()
 
 # Croniter Extentions allow you to ask for the scheduled job times, make
@@ -72,18 +77,18 @@ job3.schedule().get_prev()
 
 """
 
-import os, re, sys
+import os, re, sys, pwd
 import tempfile
 import subprocess as sp
 
-from datetime import datetime
+from datetime import date, datetime
 
 __pkgname__ = 'python-crontab'
-__version__ = '1.5.1'
+__version__ = '1.6.0'
 
-ITEMREX = re.compile('^\s*([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)' +
-    '\s+([^@#\s]+)\s+([^@#\s]+)\s+([^#\n]*)(\s+#\s*([^\n]*)|$)')
-SPECREX = re.compile('@(\w+)\s([^#\n]*)(\s+#\s*([^\n]*)|$)')
+ITEMREX = re.compile(r'^\s*([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)' +
+    r'\s+([^@#\s]+)\s+([^@#\s]+)\s+([^#\n]*)(\s+#\s*([^\n]*)|$)')
+SPECREX = re.compile(r'@(\w+)\s([^#\n]*)(\s+#\s*([^\n]*)|$)')
 DEVNULL = ">/dev/null 2>&1"
 
 MONTH_ENUM = [ None,
@@ -106,41 +111,44 @@ SPECIALS = {
     "annually": '0 0 1 1 *',
     "midnight": '0 0 * * *'
 }
+SPECIAL_IGNORE = ['midnight', 'annually']
 
 S_INFO = [
-    { 'name' : 'Minutes',      'max_v' : 59, 'min_v' : 0 },
-    { 'name' : 'Hours',        'max_v' : 23, 'min_v' : 0 },
-    { 'name' : 'Day of Month', 'max_v' : 31, 'min_v' : 1 },
-    { 'name' : 'Month',        'max_v' : 12, 'min_v' : 1, 'enum' : MONTH_ENUM },
-    { 'name' : 'Day of Week',  'max_v' : 7,  'min_v' : 0, 'enum' : WEEK_ENUM },
+    { 'name' : 'Minutes',      'max' : 59, 'min' : 0 },
+    { 'name' : 'Hours',        'max' : 23, 'min' : 0 },
+    { 'name' : 'Day of Month', 'max' : 31, 'min' : 1 },
+    { 'name' : 'Month',        'max' : 12, 'min' : 1, 'enum' : MONTH_ENUM },
+    { 'name' : 'Day of Week',  'max' : 6,  'min' : 0, 'enum' : WEEK_ENUM},
 ]
 
 # Detect Python3 and which OS for temperments.
 import platform
-py3 = platform.python_version()[0] == '3'
-WinOS = platform.system() == 'Windows'
-SystemV = not WinOS and (os.uname()[0] in ["SunOS","AIX","HP-UX"] or os.getenv('SystemV_TEST'))
+PY3 = platform.python_version()[0] == '3'
+WINOS = platform.system() == 'Windows'
+SYSTEMV = not WINOS and (os.uname()[0] in ["SunOS", "AIX", "HP-UX"] \
+          or os.getenv('SYSTEMV_TEST'))
 CRONCMD = "/usr/bin/crontab"
 if sys.argv[0].startswith('test_'):
     CRONCMD = './data/crontest'
 
-if py3:
+if PY3:
+    # pylint: disable=W0622
     unicode = str
     basestring = str
 
 try:
     # Croniter is an optional import
-    from croniter.croniter import croniter as _croniter
-    class croniter(_croniter):
+    from croniter.croniter import croniter
+    class Croniter(croniter):
         """Same as normal croniter, but always return datetime objects"""
         def get_next(self, type_ref=datetime):
-            return _croniter.get_next(self, type_ref)
+            return croniter.get_next(self, type_ref)
         def get_prev(self, type_ref=datetime):
-            return _croniter.get_prev(self, type_ref)
+            return croniter.get_prev(self, type_ref)
         def get_current(self, type_ref=datetime):
-            return _croniter.get_current(self, type_ref)
+            return croniter.get_current(self, type_ref)
 except ImportError:
-    croniter = None
+    Croniter = None
 
 
 class CronTab(object):
@@ -154,11 +162,13 @@ class CronTab(object):
 
     """
     def __init__(self, user=None, tab=None, tabfile=None, log=None):
+        if user == True and not WINOS:
+            user = pwd.getpwuid( os.getuid() )[ 0 ]
         self.lines = None
         self.crons = None
         self.filen = None
         # Protect windows users
-        self.root  = not WinOS and os.getuid() == 0
+        self.root  = not WINOS and os.getuid() == 0
         self.user  = user
         # Detect older unixes and help them out.
         self.intab = tab
@@ -167,6 +177,7 @@ class CronTab(object):
 
     @property
     def log(self):
+        """Returns the CronLog object for this tab (user or root tab only)"""
         from cronlog import CronLog
         if self._log == None or isinstance(self._log, basestring):
             self._log = CronLog(self._log, user=self.user or 'root')
@@ -186,8 +197,11 @@ class CronTab(object):
             with open(filename, 'r') as fhl:
                 lines = fhl.readlines()
         else:
-            p = sp.Popen(self._read_execute(), stdout=sp.PIPE, stderr=sp.PIPE)
-            (out, err) = p.communicate()
+            proc = sp.Popen(self._read_execute(),
+                            stdout=sp.PIPE, stderr=sp.PIPE)
+            (out, err) = proc.communicate()
+            if err:
+                raise IOError("Read crontab %s: %s" % (self.user, err))
             lines = out.decode('utf-8').split("\n")
         for line in lines:
             cron = CronItem(line, cron=self)
@@ -204,10 +218,10 @@ class CronTab(object):
 
         # Add to either the crontab or the internal tab.
         if self.intab != None:
-          self.intab = self.render()
-          # And that's it if we never saved to a file
-          if not self.filen:
-              return
+            self.intab = self.render()
+            # And that's it if we never saved to a file
+            if not self.filen:
+                return
 
         if self.filen:
             fileh = open(self.filen, 'w')
@@ -239,7 +253,7 @@ class CronTab(object):
 
         Returns the new CronItem object.
         """
-        item = CronItem(command=command, meta=comment, cron=self)
+        item = CronItem(command=command, comment=comment, cron=self)
         self.crons.append(item)
         self.lines.append(item)
         return item
@@ -248,7 +262,7 @@ class CronTab(object):
         """Return a list of crons using a command."""
         result = []
         for cron in self.crons:
-            if cron.command.match(command):
+            if command in cron.command:
                 result.append(cron)
         return result
 
@@ -256,23 +270,52 @@ class CronTab(object):
         """Return a list of crons using the comment field."""
         result = []
         for cron in self.crons:
-            if cron.meta() == comment:
+            if cron.comment == comment:
                 result.append(cron)
         return result
 
-    def remove_all(self, command):
-        """Removes all crons using the stated command."""
-        l_value = self.find_command(command)
-        for c_value in l_value:
-            self.remove(c_value)
+    @property
+    def commands(self):
+        """Return a generator of all unqiue commands used in this crontab"""
+        returned = []
+        for cron in self.crons:
+            if cron.command not in returned:
+                yield cron.command
+                returned.append(cron.command)
 
-    def remove(self, item):
+    @property
+    def comments(self):
+        """Return a generator of all unique comments/IDs used in this crontab"""
+        returned = []
+        for cron in self.crons:
+            if cron.comment and cron.comment not in returned:
+                yield cron.comment
+                returned.append(cron.comment)
+
+    def remove_all(self, command=None, comment=None):
+        """Removes all crons using the stated command OR that have the
+        stated comment OR removes everything if no arguments specified."""
+        if command:
+            return self.remove(*self.find_command(command))
+        elif comment:
+            return self.remove(*self.find_comment(comment))
+        return self.remove(*self.crons[:])
+
+    def remove(self, *items):
         """Remove a selected cron from the crontab."""
+        result = 0
+        for item in items:
+            result += self._remove(item)
+        return result
+
+    def _remove(self, item):
+        """Internal removal of an item"""
         # The last item often has a trailing line feed
         if self.crons[-1] == item and self.lines[-1] == '':
             self.lines.remove(self.lines[-1])
         self.crons.remove(item)
         self.lines.remove(item)
+        return 1
 
     def _read_execute(self):
         """Returns the command line for reading a crontab"""
@@ -294,6 +337,9 @@ class CronTab(object):
     def __unicode__(self):
         return self.render()
 
+    def __len__(self):
+        return len(self.crons)
+
     def __str__(self):
         return self.render()
 
@@ -303,68 +349,53 @@ class CronItem(object):
     An item which objectifies a single line of a crontab and
     May be considered to be a cron job object.
     """
-    def __init__(self, line=None, command='', meta='', cron=None):
-        self.valid = False
+    def __init__(self, line=None, command='', comment='', cron=None):
+        self.valid   = False
         self.enabled = True
         self.slices  = []
         self.special = False
-        self.cron  = cron
-
-        self._meta   = meta
+        self.cron    = cron
+        self.comment = comment
+        self.command = None
         self._log    = None
 
-        self.set_slices()
+        # Initalise five cron slices using static info.
+        for i_value in range(0, 5):
+            self.slices.append(CronSlice(self, S_INFO[i_value]))
+
         if line and line.strip():
             self.parse(line.strip())
 
         elif command:
-            self.command = CronCommand(unicode(command))
+            self.set_command(command)
             self.valid = True
 
     def delete(self):
+        """Delete this item and remove it from it's parent"""
         if not self.cron:
             sys.stderr.write("Cron item is not associated with a crontab!\n")
         else:
             self.cron.remove(self)
 
+    def set_command(self, cmd):
+        """Set the command and filter as needed"""
+        self.command = unicode(cmd).strip()
+
     def parse(self, line):
         """Parse a cron line string and save the info as the objects."""
         if not line or line[0] == '#':
             self.enabled = False
             line = line[1:].strip()
-        result = ITEMREX.findall(line)
-        if result:
-            o_value = result[0]
-            self.command = CronCommand(o_value[5])
-            self._meta   = o_value[7]
-            try:
-                self.set_slices( o_value )
-            except KeyError:
-                self.enabled = False
-            else:
-                self.valid = True
-        elif line.find('@') < line.find('#') or line.find('#')==-1:
-            result = SPECREX.findall(line)
-            if result and result[0][0] in SPECIALS:
-                o_value = result[0]
-                self.command = CronCommand(o_value[1])
-                self._meta   = o_value[3]
-                value = SPECIALS[o_value[0]]
-                if value.find('@') != -1:
-                    self.special = value
-                else:
-                    self.set_slices( value.split(' ') )
-                self.valid = True
+        self._set_parse(ITEMREX.findall(line))
+        self._set_parse(SPECREX.findall(line))
 
-    def set_slices(self, o_value=None):
-        """Set the values of this slice set"""
-        self.slices = []
-        for i_value in range(0, 5):
-            if not o_value:
-                o_value = [None, None, None, None, None]
-            self.slices.append(
-                CronSlice(job=self, value=o_value[i_value],
-                    **S_INFO[i_value]))
+    def _set_parse(self, result):
+        if not result:
+            return
+        self.set_command(result[0][-3])
+        self.comment = result[0][-1]
+        self.valid   = self.setall(*result[0][:-3])
+        self.enabled = self.enabled and self.valid
 
     def enable(self, enabled=True):
         """Set if this cron job is enabled or not"""
@@ -387,30 +418,23 @@ class CronItem(object):
     def render_schedule(self):
         """Return just the first part of a cron job (the numbers or specials)"""
         time = self.render_time()
-        if self.special or (time in SPECIALS.values() and not SystemV):
-            if self.special:
-                return self.special
-            else:
-                for (name, value) in SPECIALS.items():
-                    if value == time:
-                        return "@%s" % name
+        if self.special:
+            return self.special
+        elif not SYSTEMV:
+            for (name, value) in SPECIALS.items():
+                if value == time and name not in SPECIAL_IGNORE:
+                    return "@%s" % name
         return time
 
     def render(self):
         """Render this set cron-job to a string"""
-        result = "%s %s" % (self.render_schedule(), unicode(self.command))
-        if self.meta():
-            result += " # " + self.meta()
+        result = "%s %s" % (self.render_schedule(), self.command)
+        if self.comment:
+            result += " # " + self.comment
         if not self.enabled:
             result = "# " + result
         return result
 
-    def meta(self, value=None):
-        """Return or set the meta value to replace the set values"""
-        if value:
-            self._meta = value
-        return self._meta
-
     def every_reboot(self):
         """Set to every reboot instead of a time pattern: @reboot"""
         self.clear()
@@ -429,18 +453,73 @@ class CronItem(object):
         """
         return SimpleItemInterface(self, unit)
 
+    def setall(self, *args):
+        """Replace existing time pattern with these five values given as args:
+
+           job.setall("1 2 * * *")
+           job.setall(1, 2) == '1 2 * * *'
+           job.setall(0, 0, None, '>', 'SUN') == '0 0 * 12 SUN'
+        """
+        if len(args) == 1 and isinstance(args[0], basestring):
+            if args[0].count(' ') == 4:
+                args = args[0].strip().split(' ')
+            elif args[0] == 'reboot':
+                self.special = '@'+args[0]
+                return True
+            elif args[0].strip()[0] == '@':
+                self.setall(SPECIALS.get(args[0][1:], args[0][1:]))
+
+        for x, s in enumerate(self.slices):
+            try:
+                s.parse(args[x])
+            except IndexError:
+                s.clear()
+            except KeyError:
+                return False
+        return True
+
     def clear(self):
         """Clear the special and set values"""
         self.special = None
         for slice_v in self.slices:
             slice_v.clear()
 
+    def frequency(self, year=None):
+        """Returns the number of times this item will execute in a given year
+           (defaults to this year)
+        """
+        return self.frequency_per_year() * self.frequency_per_day()
+
+    def frequency_per_year(self, year=None):
+        """Returns the number of /days/ this item will execute on in a year
+           (defaults to this year)
+        """
+        result = 0
+        if not year:
+            year = date.today().year
+
+        weekdays = list(self[4])
+        days = (date(year+1,1,1) - date(year,1,1)).days
+
+        for month in self[3]:
+            for day in self[2]:
+                try:
+                    if date(year,month,day).weekday() in weekdays:
+                        result += 1
+                except ValueError:
+                    continue
+        return result
+
+    def frequency_per_day(self):
+        """Returns the number of time this item will execute in any day"""
+        return len(self[0]) * len(self[1])
+
     def schedule(self, date_from=None):
         """Return a croniter schedule is available."""
         if not date_from:
             date_from = datetime.now()
-        if croniter:
-            return croniter(self.render_time(), date_from)
+        if Croniter:
+            return Croniter(self.render_time(), date_from)
         raise ImportError("Croniter is not available. Please install croniter"+\
                          " python module via pip or your package manager")
 
@@ -471,6 +550,10 @@ class CronItem(object):
         """Same as hour"""
         return self.hour
 
+    @property
+    def day(self):
+        return self.dom
+
     @property
     def dom(self):
         """Return the day-of-the month slice"""
@@ -482,7 +565,7 @@ class CronItem(object):
         return self.slices[3]
 
     @property
-    def months(Self):
+    def months(self):
         """Same as month"""
         return self.month
 
@@ -494,6 +577,12 @@ class CronItem(object):
     def __repr__(self):
         return "<CronJob '%s'>" % str(self)
 
+    def __len__(self):
+        return len(str(self))
+
+    def __getitem__(self, x):
+        return self.slices[x]
+
     def __eq__(self, value):
         return str(self) == str(value)
 
@@ -518,12 +607,14 @@ class SimpleItemInterface(object):
     def __init__(self, item, units):
         self.job = item
         self.unit = units
-        for (x, i) in enumerate(['minute', 'hour', 'dom', 'month', 'dow']):
-            setattr(self, i, self._set(x))
-            setattr(self, i+'s', self._set(x))
+        for (x, i) in enumerate(['minute', 'hour', 'dom', 'month', 'dow',
+                                 'min', 'hour', 'day', 'moon', 'weekday']):
+            setattr(self, i, self._set(x % 5))
+            setattr(self, i+'s', self._set(x % 5))
 
     def _set(self, target):
         def innercall():
+            """Returned inner call for setting slice targets"""
             self.job.clear()
             # Day-of-week is actually a level 2 set, not level 4.
             for p in range(target == 4 and 2 or target):
@@ -542,20 +633,20 @@ class SimpleItemInterface(object):
 
 class CronSlice(object):
     """Cron slice object which shows a time pattern"""
-    def __init__(self, name, min_v, max_v, enum=None, value=None, job=None):
-        self.name  = name
-        self.min   = min_v
-        self.max   = max_v
+    def __init__(self, job, info, value=None):
         self.job   = job
-        self.enum  = enum
+        self.name  = info.get('name', None)
+        self.min   = info.get('min', None)
+        self.max   = info.get('max', None)
+        self.enum  = info.get('enum', None)
         self.parts = []
         if value:
-            self._set_value(value)
+            self.parse(value)
 
-    def _set_value(self, value):
+    def parse(self, value):
         """Set values into the slice."""
         self.parts = []
-        for part in value.split(','):
+        for part in str(value).split(','):
             if part.find("/") > 0 or part.find("-") > 0 or part == '*':
                 self.parts.append( self.get_range( part ) )
             else:
@@ -589,32 +680,41 @@ class CronSlice(object):
 
     def every(self, n_value, also=False):
         """Set the every X units value"""
-        not also and self.clear()
+        if not also:
+            self.clear()
         self.parts.append( self.get_range( int(n_value) ) )
         return self.parts[-1]
 
     def on(self, *n_value, **opts):
         """Set the time values to the specified placements."""
-        not opts.get('also', False) and self.clear()
+        if not opts.get('also', False):
+            self.clear()
         for av in n_value:
             self.parts += self._v(av),
         return self.parts
 
     def during(self, vfrom, vto, also=False):
         """Set the During value, which sets a range"""
-        not also and self.clear()
+        if not also:
+            self.clear()
         self.parts.append(self.get_range(self._v(vfrom), self._v(vto)))
         return self.parts[-1]
 
     @property
     def also(self):
+        """Appends rather than replaces the new values"""
+        outself = self
         class Also(object):
-            def every(s, *a):
-                return self.every(*a, also=True)
-            def on(s, *a):
-                return self.on(*a, also=True)
-            def during(s, *a):
-                return self.during(*a, also=True)
+            """Will append new values"""
+            def every(self, *a):
+                """Also every one of these"""
+                return outself.every(*a, also=True)
+            def on(self, *a):
+                """Also on these"""
+                return outself.on(*a, also=True)
+            def during(self, *a):
+                """Also during these"""
+                return outself.during(*a, also=True)
         return Also()
 
     def clear(self):
@@ -625,8 +725,23 @@ class CronSlice(object):
         """Return a cron range for this slice"""
         return CronRange( self, *vrange )
 
+    def __iter__(self):
+        """Return the entire element as an iterable"""
+        r = {}
+        for part in self.parts:
+            if isinstance(part, CronRange):
+                for bit in part.range():
+                    r[bit] = 1
+            else:
+                r[int(part)] = 1
+        for x in r:
+            yield x
+
+    def __len__(self):
+        """Returns the number of times this slice happens in it's range"""
+        return len(list(self.__iter__()))
+
     def _v(self, v):
-        """Support wrapper for enumerations and check for range"""
         if v == '>':
             v = self.max
         elif v == '<':
@@ -638,11 +753,19 @@ class CronSlice(object):
         except KeyError:
             raise KeyError("No enumeration '%s' got '%s'" % (self.name, v))
 
+        # If it's a sunday, make it a sunday
+        if self.max == 6 and v == 7:
+            out = 0
+
         if int(out) < self.min or int(out) > self.max:
             raise ValueError("Invalid value '%s', expected %d-%d for %s" % (
                 str(v), self.min, self.max, self.name))
         return out
 
+    def filter_v(self, v):
+        """Support wrapper for enumerations and check for range"""
+        return self._v(v)
+
 
 def get_cronvalue(value, enums):
     """Returns a value as int (pass-through) or a special enum value"""
@@ -685,8 +808,6 @@ def _render(value, resolve=False):
         return str(int(value))
     return str(value)
 
-
-
 class CronRange(object):
     """A range between one value and another for a time range."""
     def __init__(self, vslice, *vrange):
@@ -707,17 +828,17 @@ class CronRange(object):
 
     def parse(self, value):
         """Parse a ranged value in a cronjob"""
-        if value.find('/') > 0:
+        if value.count('/') == 1:
             value, seq = value.split('/')
             self.seq = int(seq)
-        if value.find('-') > 0:
+        if value.count('-') == 1:
             vfrom, vto = value.split('-')
-            self.vfrom = self.slice._v(vfrom)
-            self.vto  = self.slice._v(vto)
+            self.vfrom = self.slice.filter_v(vfrom)
+            self.vto  = self.slice.filter_v(vto)
         elif value == '*':
             self.all()
         else:
-            raise ValueError('Unknown cron range value %s' % value)
+            raise ValueError('Unknown cron range value "%s"' % value)
 
     def all(self):
         """Set this slice to all units between the miniumum and maximum"""
@@ -731,10 +852,14 @@ class CronRange(object):
             value = _render_values([self.vfrom, self.vto], '-', resolve)
         if self.seq != 1:
             value += "/%d" % self.seq
-        if value != '*' and SystemV:
-            value = ','.join(map(str, range(self.vfrom, self.vto+1, self.seq)))
+        if value != '*' and SYSTEMV:
+            value = ','.join(map(str, self.range()))
         return value
 
+    def range(self):
+        """Returns the range of this cron slice as a iterable list"""
+        return range(self.vfrom, self.vto+1, self.seq)
+
     def every(self, value):
         """Set the sequence value for this range."""
         self.seq = int(value)
@@ -742,6 +867,9 @@ class CronRange(object):
     def __lt__(self, value):
         return int(self.vfrom) < int(value)
 
+    def __gt__(self, value):
+        return int(self.vto) > int(value)
+
     def __int__(self):
         return int(self.vfrom)
 
@@ -750,28 +878,3 @@ class CronRange(object):
 
     def __unicode__(self):
         return self.render()
-
-
-class CronCommand(object):
-    """Represent a cron command as an object."""
-    def __init__(self, line):
-        self._command = line
-
-    def match(self, command):
-        """Match the command given"""
-        return command in self._command
-
-    def command(self):
-        """Return the command line"""
-        return self._command
-
-    def __str__(self):
-        """Return a string as a value"""
-        # TODO: encode with the system's default encoding?
-        # i.e. locale.getpreferredencoding()
-        return self.__unicode__()
-
-    def __unicode__(self):
-        """Return unicode command line value"""
-        return self.command().strip()
-
