@@ -26,13 +26,13 @@ import sys
 sys.path.insert(0, '../')
 
 import unittest
-from crontab import CronTab, py3
+from crontab import CronTab, PY3
 try:
     from test import test_support
 except ImportError:
     from test import support as test_support
 
-if py3:
+if PY3:
     unicode = str
 
 class EveryTestCase(unittest.TestCase):
