@@ -26,13 +26,13 @@ import sys
 sys.path.insert(0, '../')
 
 import unittest
-from crontab import CronTab, py3
+from crontab import CronTab, PY3
 try:
     from test import test_support
 except ImportError:
     from test import support as test_support
 
-if py3:
+if PY3:
     unicode = str
 
 COMMANDS = [
@@ -55,8 +55,8 @@ RESULT_TAB = """# First Comment
 0 5 * * * spaced # Comment  Two
 
 
-@reboot rebooted
-# Last Comment
+@reboot rebooted # re-id
+# Last Comment @has this # extra
 """
 
 class BasicTestCase(unittest.TestCase):
@@ -104,7 +104,7 @@ class BasicTestCase(unittest.TestCase):
         job.month.on(6)
         self.assertEqual(job.render(), '* 4 5 6 * fields')
         job.dow.on(7)
-        self.assertEqual(job.render(), '* 4 5 6 7 fields')
+        self.assertEqual(job.render(), '* 4 5 6 0 fields')
 
     def test_06_clear(self):
         """Render Hours Days and Weeks"""
@@ -114,7 +114,7 @@ class BasicTestCase(unittest.TestCase):
         job.dom.on(5)
         job.month.on(6)
         job.dow.on(7)
-        self.assertEqual(job.render(), '3 4 5 6 7 clear')
+        self.assertEqual(job.render(), '3 4 5 6 0 clear')
         job.clear()
         self.assertEqual(job.render(), '* * * * * clear')
 
@@ -205,7 +205,28 @@ class BasicTestCase(unittest.TestCase):
     def test_24_specials(self):
         """Specials Conversion"""
         tab = CronTab(tabfile='data/specials.tab')
-        self.assertEqual(tab.render(), """@hourly hourly\n@midnight daily\n@weekly weekly\n""")
+        self.assertEqual(tab.render(), """@hourly hourly\n@daily daily\n@weekly weekly\n""")
+
+    def test_25_setall(self):
+        """Set all values at once"""
+        job = self.crontab.new(command='all')
+        job.setall(1, '*/2', '2-4', '>', 'SUN')
+        self.assertEqual(unicode(job), '1 */2 2-4 12 SUN all')
+        job.setall()
+        self.assertEqual(unicode(job), '* * * * * all')
+        job.setall('1 */2 2-4 12 SUN')
+        self.assertEqual(unicode(job), '1 */2 2-4 12 SUN all')
+
+    def test_26_commands(self):
+        """Get all commands"""
+        self.assertEqual(list(self.crontab.commands), 
+                         [u'firstcommand', u'range', u'byweek',
+                          u'disabled', u'spaced', u'rebooted'])
+
+    def test_27_comments(self):
+        """Get all comments"""
+        self.assertEqual(list(self.crontab.comments),
+                         ['Comment One', 'Comment  Two', 're-id'])
 
 if __name__ == '__main__':
     test_support.run_unittest(
